//===================================================================================
// Microsoft patterns & practices
// Composite Application Guidance for Windows Presentation Foundation and Silverlight
//===================================================================================
// Copyright (c) Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===================================================================================
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
//===================================================================================
using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Windows;
using System.Windows.Controls;
using Microsoft.Practices.Composite.Presentation.Properties;
using Microsoft.Practices.Composite.Presentation.Regions;
using Microsoft.Practices.Composite.Presentation.Regions.Behaviors;
using Microsoft.Practices.Composite.Regions;
using Telerik.Windows.Controls;

namespace Telerik.Regions
{
    /// <summary>
    /// Adapter that creates a new <see cref="Region"/> and binds all
    /// the views to the adapted <see cref="RadTabControl"/>.
    /// </summary>
    /// <remarks>
    /// This adapter is needed on Silverlight because the <see cref="RadTabControl"/> doesn't 
    /// automatically create <see cref="RadTabItem"/>s when new items are added to 
    /// the <see cref="ItemsControl.Items"/> collection.
    /// </remarks>
    public class RadTabControlRegionAdapter : RegionAdapterBase<RadTabControl>
    {
        /// <summary>
        /// <see cref="Style"/> to set to the created <see cref="RadTabItem"/>.
        /// </summary>
        public static readonly DependencyProperty ItemContainerStyleProperty =
            DependencyProperty.RegisterAttached("ItemContainerStyle", typeof(Style), typeof(RadTabControlRegionAdapter), null);

        /// <summary>
        /// Initializes a new instance of the <see cref="RadTabControlRegionAdapter"/> class.
        /// </summary>
        /// <param name="regionBehaviorFactory">The factory used to create the region behaviors to attach to the created regions.</param>
        public RadTabControlRegionAdapter(IRegionBehaviorFactory regionBehaviorFactory)
            : base(regionBehaviorFactory)
        {
        }

        /// <summary>
        /// Gets the <see cref="ItemContainerStyleProperty"/> property value.
        /// </summary>
        /// <param name="target">Target object of the attached property.</param>
        /// <returns>Value of the <see cref="ItemContainerStyleProperty"/> property.</returns>
        public static Style GetItemContainerStyle(DependencyObject target)
        {
            return (Style)target.GetValue(ItemContainerStyleProperty);
        }

        /// <summary>
        /// Sets the <see cref="ItemContainerStyleProperty"/> property value.
        /// </summary>
        /// <param name="target">Target object of the attached property.</param>
        /// <param name="value">Value to be set on the <see cref="ItemContainerStyleProperty"/> property.</param>
        public static void SetItemContainerStyle(DependencyObject target, Style value)
        {
            target.SetValue(ItemContainerStyleProperty, value);
        }

        /// <summary>
        /// Adapts a <see cref="RadTabControl"/> to an <see cref="IRegion"/>.
        /// </summary>
        /// <param name="region">The new region being used.</param>
        /// <param name="regionTarget">The object to adapt.</param>
        protected override void Adapt(IRegion region, RadTabControl regionTarget)
        {
            bool itemsSourceIsSet = regionTarget.ItemsSource != null;

            if (itemsSourceIsSet)
            {
                throw new InvalidOperationException("ItemsControlHasItemsSourceException");
            }
        }

        /// <summary>
        /// Attach new behaviors.
        /// </summary>
        /// <param name="region">The region being used.</param>
        /// <param name="regionTarget">The object to adapt.</param>
        /// <remarks>
        /// This class attaches the base behaviors and also keeps the <see cref="RadTabControl.SelectedItem"/> 
        /// and the <see cref="IRegion.ActiveViews"/> in sync.
        /// </remarks>
        protected override void AttachBehaviors(IRegion region, RadTabControl regionTarget)
        {
            base.AttachBehaviors(region, regionTarget);
            region.Behaviors.Add(RadTabControlRegionSyncBehavior.BehaviorKey, new RadTabControlRegionSyncBehavior { HostControl = regionTarget });
        }

        /// <summary>
        /// Creates a new instance of <see cref="Region"/>.
        /// </summary>
        /// <returns>A new instance of <see cref="Region"/>.</returns>
        protected override IRegion CreateRegion()
        {
            return new SingleActiveRegion();
        }

        /// <summary>
        /// Behavior that generates <see cref="RadTabItem"/> containers for the added items
        /// and also keeps the <see cref="RadTabControl.SelectedItem"/> and the <see cref="IRegion.ActiveViews"/> in sync.
        /// </summary>
        private class RadTabControlRegionSyncBehavior : RegionBehavior, IHostAwareRegionBehavior
        {
            public const string BehaviorKey = "RadTabControlRegionSyncBehavior";

            private static readonly DependencyProperty IsGeneratedProperty =
                DependencyProperty.RegisterAttached("IsGenerated", typeof(bool), typeof(RadTabControlRegionSyncBehavior), null);

            private RadTabControl hostControl;

            public DependencyObject HostControl
            {
                get
                {
                    return this.hostControl;
                }

                set
                {
                    RadTabControl newValue = value as RadTabControl;
                    if (newValue == null)
                    {
                        throw new InvalidOperationException("HostControlMustBeARadTabControl");
                    }

                    if (IsAttached)
                    {
                        throw new InvalidOperationException("HostControlCannotBeSetAfterAttach");
                    }

                    this.hostControl = newValue;
                }
            }

            protected override void OnAttach()
            {
                if (this.hostControl == null)
                {
                    throw new InvalidOperationException("HostControlCannotBeNull");
                }

                this.SynchronizeItems();

                //TODO: this line comment because the different from Telerik RoutedEventArgs and SelectionChangedEventArgs of MS Tab Control
                //this.hostControl.SelectionChanged += this.OnSelectionChanged;

                this.Region.ActiveViews.CollectionChanged += this.OnActiveViewsChanged;
                this.Region.Views.CollectionChanged += this.OnViewsChanged;
            }

            void hostControl_SelectionChanged(object sender, RoutedEventArgs e)
            {

            }

            private static object GetContainedItem(RadTabItem RadTabItem)
            {
                if ((bool)RadTabItem.GetValue(IsGeneratedProperty))
                {
                    return RadTabItem.Content;
                }

                return RadTabItem;
            }

            private static RadTabItem PrepareContainerForItem(object item, DependencyObject parent)
            {
                RadTabItem container = item as RadTabItem;
                if (container == null)
                {
                    container = new RadTabItem();
                    container.Content = item;
                    container.DataContext = GetDataContext(item);
                    container.Style = GetItemContainerStyle(parent);
                    container.SetValue(IsGeneratedProperty, true);
                }

                return container;
            }

            private static void ClearContainerForItem(RadTabItem RadTabItem)
            {
                if ((bool)RadTabItem.GetValue(IsGeneratedProperty))
                {
                    RadTabItem.Content = null;
                }
            }

            /// <summary>
            /// Return the appropriate data context.  If the item is a FrameworkElement it cannot be a data context in Silverlight, so we use its data context.
            /// Otherwise, we just us the item as the data context.
            /// </summary>
            /// <param name="item"></param>
            /// <returns></returns>
            private static object GetDataContext(object item)
            {
                FrameworkElement frameworkElement = item as FrameworkElement;
                return frameworkElement == null ? item : frameworkElement.DataContext;
            }

            private static RadTabItem GetContainerForItem(object item, ItemCollection itemCollection)
            {
                RadTabItem container = item as RadTabItem;
                if (container != null && ((bool)container.GetValue(IsGeneratedProperty)) == false)
                {
                    return container;
                }

                foreach (RadTabItem RadTabItem in itemCollection)
                {
                    if ((bool)RadTabItem.GetValue(IsGeneratedProperty))
                    {
                        if (RadTabItem.Content == item)
                        {
                            return RadTabItem;
                        }
                    }
                }

                return null;
            }

            private void SynchronizeItems()
            {
                List<object> existingItems = new List<object>();
                if (this.hostControl.Items.Count > 0)
                {
                    // Control must be empty before "Binding" to a region
                    foreach (object childItem in this.hostControl.Items)
                    {
                        existingItems.Add(childItem);
                    }
                }

                foreach (object view in this.Region.Views)
                {
                    RadTabItem RadTabItem = PrepareContainerForItem(view, this.hostControl);
                    this.hostControl.Items.Add(RadTabItem);
                }

                foreach (object existingItem in existingItems)
                {
                    this.Region.Add(existingItem);
                }
            }

            private void OnSelectionChanged(object sender, Telerik.Windows.Controls.SelectionChangedEventArgs e)
            {
                // e.OriginalSource == null, that's why we use sender.
                if (this.hostControl == sender)
                {

                    foreach (RadTabItem RadTabItem in e.RemovedItems)
                    {
                        object item = GetContainedItem(RadTabItem);

                        // check if the view is in both Views and ActiveViews collections (there may be out of sync)
                        if (this.Region.Views.Contains(item) && this.Region.ActiveViews.Contains(item))
                        {
                            this.Region.Deactivate(item);
                        }
                    }

                    foreach (RadTabItem RadTabItem in e.AddedItems)
                    {
                        object item = GetContainedItem(RadTabItem);
                        if (!this.Region.ActiveViews.Contains(item))
                        {
                            this.Region.Activate(item);
                        }
                    }
                }
            }

            private void OnActiveViewsChanged(object sender, NotifyCollectionChangedEventArgs e)
            {
                if (e.Action == NotifyCollectionChangedAction.Add)
                {
                    this.hostControl.SelectedItem = GetContainerForItem(e.NewItems[0], this.hostControl.Items);
                }
                else if (e.Action == NotifyCollectionChangedAction.Remove
                    && this.hostControl.SelectedItem != null
                    && e.OldItems.Contains(GetContainedItem((RadTabItem)this.hostControl.SelectedItem)))
                {
                    this.hostControl.SelectedItem = null;
                }
            }

            private void OnViewsChanged(object sender, NotifyCollectionChangedEventArgs e)
            {
                if (e.Action == NotifyCollectionChangedAction.Add)
                {
                    foreach (object newItem in e.NewItems)
                    {
                        RadTabItem RadTabItem = PrepareContainerForItem(newItem, this.hostControl);
                        this.hostControl.Items.Add(RadTabItem);
                    }
                }
                else if (e.Action == NotifyCollectionChangedAction.Remove)
                {
                    foreach (object oldItem in e.OldItems)
                    {
                        RadTabItem RadTabItem = GetContainerForItem(oldItem, this.hostControl.Items);
                        this.hostControl.Items.Remove(RadTabItem);
                        ClearContainerForItem(RadTabItem);
                    }
                }
            }
        }
    }
}
